#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_security.interfaces import ISecurityManager
from pyams_utils.interfaces.site import ISiteGenerations
from zope.lifecycleevent.interfaces import IObjectCreatedEvent
from zope.principalannotation.interfaces import IPrincipalAnnotationUtility
from zope.site.interfaces import INewLocalSite

# import packages
from pyams_security.plugin.admin import AdminAuthenticationPlugin
from pyams_security.utility import SecurityManager
from pyams_utils.registry import utility_config
from pyams_utils.site import check_required_utilities
from pyramid.events import subscriber
from pyramid.threadlocal import get_current_registry
from zope.lifecycleevent import ObjectCreatedEvent
from zope.principalannotation.utility import PrincipalAnnotationUtility


REQUIRED_UTILITIES = ((ISecurityManager, '', SecurityManager, 'Security manager'),
                      (IPrincipalAnnotationUtility, '', PrincipalAnnotationUtility, 'User profiles'))


@subscriber(INewLocalSite)
def handle_new_local_site(event):
    """Create a new security manager when a site is created"""
    site = event.manager.__parent__
    check_required_utilities(site, REQUIRED_UTILITIES)


@utility_config(name='PyAMS security', provides=ISiteGenerations)
class SecurityGenerationsChecker(object):
    """I18n generations checker"""

    generation = 1

    def evolve(self, site, current=None):
        """Check for required utilities"""
        check_required_utilities(site, REQUIRED_UTILITIES)


@subscriber(IObjectCreatedEvent, context_selector=ISecurityManager)
def handle_new_security_manager(event):
    """Automatically create a new administration login"""
    admin_auth = AdminAuthenticationPlugin()
    admin_auth.prefix = 'system'
    admin_auth.title = 'System manager authentication'
    admin_auth.login = 'admin'
    admin_auth.password = 'admin'
    get_current_registry().notify(ObjectCreatedEvent(admin_auth))
    utility = event.object
    utility['__system__'] = admin_auth
