#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_security.interfaces import ISecurityManager
from pyams_security.interfaces.notification import INotificationSettings
from pyams_security.zmi.interfaces import ISecurityManagerMenu
from pyams_skin.layer import IPyAMSLayer
from pyams_zmi.layer import IAdminLayer
from zope.component.interfaces import ISite

# import packages
from pyams_form.form import AJAXEditForm
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.viewlet.menu import MenuItem
from pyams_utils.registry import query_utility
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogEditForm
from pyramid.url import resource_url
from pyramid.view import view_config
from z3c.form import field

from pyams_security import _


@viewlet_config(name='notifications.menu', context=ISite, layer=IAdminLayer, manager=ISecurityManagerMenu,
                permission='system.view', weight=2)
class SecurityManagerNotificationsMenuItem(MenuItem):
    """Notifications properties menu item"""

    label = _("Notifications...")
    url = 'notifications.html'
    modal_target = True

    def get_url(self):
        manager = query_utility(ISecurityManager)
        return resource_url(manager, self.request, self.url)


@pagelet_config(name='notifications.html', context=ISecurityManager, layer=IPyAMSLayer, permission='system.view')
class SecurityManagerNotificationsEditForm(AdminDialogEditForm):
    """Security manager notifications edit form"""

    title = _("System security manager")
    legend = _("Notifications properties")
    icon_css_class = 'fa fa-fw fa-envelope-o'
    label_css_class = 'control-label col-md-5'
    input_css_class = 'col-md-7'

    fields = field.Fields(INotificationSettings)
    ajax_handler = 'notifications.json'

    def getContent(self):
        return query_utility(ISecurityManager)

    def updateWidgets(self, prefix=None):
        super(SecurityManagerNotificationsEditForm, self).updateWidgets(prefix)
        self.widgets['signature'].label_css_class = 'textarea'


@view_config(name='notifications.json', context=ISecurityManager, request_type=IPyAMSLayer,
             permission='system.manage', renderer='json', xhr=True)
class SecurityManagerNotificationsAJAXEditForm(AJAXEditForm, SecurityManagerNotificationsEditForm):
    """Security manager notifications edit form, AJAX view"""
